@@ -26,9 +26,16 @@ def xlogy0(m, x):
     return at.switch(at.eq(x, 0), at.switch(at.eq(m, 0), 0.0, -np.inf), m * at.log(x))
 
 
-def logprob(rv_var, obs, **kwargs):
+def logprob(rv_var, rv_value, **kwargs):
     """Create a graph for the log-probability of a ``RandomVariable``."""
-    return _logprob(rv_var.owner.op, obs, *rv_var.owner.inputs, **kwargs)
+    logprob = _logprob(
+        rv_var.owner.op, rv_value, *rv_var.owner.inputs, name=rv_var.name, **kwargs
+    )
+
+    if rv_var.name:
+        logprob.name = f"{rv_var.name}_logprob"
+
+    return logprob
 
 
 @singledispatch
@@ -44,8 +51,6 @@ def _logprob(
     of ``RandomVariable``.  If you want to implement new density/mass graphs
     for a ``RandomVariable``, register a new function on this dispatcher.
 
-    The default returns a graph producing only zeros.
-
     """
     raise NotImplementedError()
 
@@ -190,6 +195,29 @@ def invgamma_logprob(op, value, *inputs, **kwargs):
     return res
 
 
+@_logprob.register(arb.ChiSquareRV)
+def chisquare_logprob(op, value, *inputs, **kwargs):
+    (nu,) = inputs[3:]
+    res = gamma_logprob(op, value, *(*inputs[:3], nu / 2, 2))
+    return res
+
+
+@_logprob.register(arb.WaldRV)
+def wald_logprob(op, value, *inputs, **kwargs):
+    mu, scale = inputs[3:]
+    res = (
+        0.5 * at.log(scale / (2.0 * np.pi))
+        - 1.5 * at.log(value)
+        - 0.5 * scale / value * ((value - mu) / mu) ** 2
+    )
+
+    res = at.switch(at.gt(value, 0.0), res, -np.inf)
+    res = Assert("mu > 0, scale > 0")(
+        res, at.all(at.gt(mu, 0.0)), at.all(at.gt(scale, 0.0))
+    )
+    return res
+
+
 @_logprob.register(arb.WeibullRV)
 def weibull_logprob(op, value, *inputs, **kwargs):
     alpha, beta = inputs[3:]
@@ -344,43 +372,30 @@ def hypergeometric_logprob(op, value, *inputs, **kwargs):
 def categorical_logprob(op, value, *inputs, **kwargs):
     (p,) = inputs[3:]
 
-    raise NotImplementedError()
-
-    p_ = p
-    p = p_ / at.sum(p_, axis=-1, keepdims=True)
-
-    # FIXME: Why waste a `clip` on these?  If they're out of range, that's
-    # simply an error/invalid inputs.  Use `Assert` instead.
-    k = at.shape(p)[-1]
-    value_clip = at.clip(value, 0, k - 1)
+    p = p / at.sum(p, axis=-1, keepdims=True)
 
     if p.ndim > 1:
-        # FIXME: This could probably be done much easier with something like `at.ogrid`.
-        # E.g. something like
-        # ind_slices = tuple(at.ogrid[[slice(None, d) for d in tuple(p.shape)[:-1]]])
-        # However, if that produces an `AdvancedSubtensor*` and this approach doesn't,
-        # then this one is probably better.
-        pass
+        if p.ndim > value.ndim:
+            value = at.shape_padleft(value, p.ndim - value.ndim)
+        elif p.ndim < value.ndim:
+            p = at.shape_padleft(p, value.ndim - p.ndim)
 
-        # if p.ndim > value_clip.ndim:
-        #     value_clip = at.shape_padleft(value_clip, p_.ndim - value_clip.ndim)
-        # elif p.ndim < value_clip.ndim:
-        #     p = at.shape_padleft(p, value_clip.ndim - p_.ndim)
-        # pattern = (p.ndim - 1,) + tuple(range(p.ndim - 1))
-        #
-        # res = at.log(
-        #     take_along_axis(
-        #         p.dimshuffle(pattern),
-        #         value_clip,
-        #     )
-        # )
+        pattern = (p.ndim - 1,) + tuple(range(p.ndim - 1))
+        res = at.log(
+            at.take_along_axis(
+                p.dimshuffle(pattern),
+                value,
+            )
+        )
+        # FIXME: `take_along_axis` drops a broadcastable dimension
+        # when `value.broadcastable == p.broadcastable == (True, True, False)`.
     else:
-        res = at.log(p[value_clip])
+        res = at.log(p[value])
 
-    res = at.switch(at.bitwise_and(at.le(0, value), at.le(value, k - 1)), res, -np.inf)
-    res = Assert("0 <= p <= 1")(
-        res, at.all(at.ge(p_, 0.0), axis=-1), at.all(at.le(p, 1.0), axis=-1)
+    res = at.switch(
+        at.bitwise_and(at.le(0, value), at.lt(value, at.shape(p)[-1])), res, -np.inf
     )
+    res = Assert("0 <= p <= 1")(res, at.all(at.ge(p, 0.0)), at.all(at.le(p, 1.0)))
     return res
 
 
@@ -421,9 +436,6 @@ def dirichlet_logprob(op, value, *inputs, **kwargs):
     (alpha,) = inputs[3:]
     res = at.sum(at.gammaln(alpha)) - at.gammaln(at.sum(alpha))
     res = -res + at.sum((xlogy0(alpha - 1, value.T)).T, axis=0)
-    # res = at.sum(logpow(value, alpha - 1) - at.gammaln(alpha), axis=-1) + at.gammaln(
-    #     at.sum(alpha, axis=-1)
-    # )
     res = at.switch(
         at.bitwise_and(
             at.all(at.le(0.0, value), axis=-1), at.all(at.le(value, 1.0), axis=-1)
